from enum import Enum
from dataclasses import dataclass

from .subtypes import Adress, Company


class ApiCategories(Enum):
    todos = "todos"
    users = "users"
    posts = "posts"
    comments = "comments"
    albums = "albums"
    photos = "photos"


@dataclass
class Todo:
    user_d: int
    id: int
    title: str
    completed: bool


@dataclass
class User:
    id: int
    name: str
    username: str
    email: str
    adress: Adress
    phone: str
    website: str
    company: Company


@dataclass
class Photo:
    album_id: int
    id: int
    title: str
    url: str
    thumbnail_url: str


@dataclass
class Album:
    user_id: int
    id: int
    title: str


@dataclass
class Comment:
    post_id: int
    id: int
    name: str
    email: str
    body: str


@dataclass
class Post:
    user_id: int
    id: int
    title: str
    body: str



from .apputils import *


class PlaceholderApi:
    """Class for interacting with Jsonplaceholder"""
    
    def __init__(self) -> None:
        self.__base_url = "https://jsonplaceholder.typicode.com/"


    def todos(self, count: int = 1):
        """Max count: 200"""
        if count > 200:
            return "The number of data is less than 200"
        else:
            return get_todos(self.__base_url, count)

    def users(self, count: int = 1):
        """Max count: 10"""
        if count > 10:
            return "The number of data is less than 10"
        else:
            return get_users(self.__base_url, count)

    def posts(self, count: int = 1):
        """Max count: 100"""
        if count > 100:
            return "The number of data is less than 100"
        else:
            return get_posts(self.__base_url, count)

    def photos(self, count: int = 1):
        """Max count: 5000"""
        if count > 5000:
            return "The number of data is less than 5000"
        else:
            return get_photos(self.__base_url, count)

    def albums(self, count: int = 1):
        """Max count: 100"""
        if count > 100:
            return "The number of data is less than 100"
        else:
            return get_albums(self.__base_url, count)

    def comments(self, count: int = 1):
        """Max count: 500"""
        if count > 500:
            return "The number of data is less than 500"
        else:
            return get_comments(self.__base_url, count)


from requests import get

from .apptypes import ApiCategories, Todo, User, Post, Photo, Comment, Album
from .subtypes import Adress, Geo, Company


def build_request_url(url: str, category: ApiCategories):
    return url + category.value


def get_todos(url: str, count: int):
    result: list[Todo] = []
    request_url = build_request_url(url, ApiCategories.todos)

    response = get(request_url).json()

    for i in range(0, count):
        result.append(Todo(
            user_id=response[i]['userId'],
            id=response[i]['id'],
            title=response[i]['title'],
            completed=response[i]['completed']
        ))

    return result


def get_users(url: str, count: int):
    result: list[User] = []
    request_url = build_request_url(url, ApiCategories.users)

    response = get(request_url).json()

    for i in range(0, count):
        result.append(User(
            id=response[i]['id'],
            name=response[i]['name'],
            username=response[i]['username'],
            email=response[i]['email'],
            adress=Adress(
                street=response[i]['address']['street'],
                suite=response[i]['address']['suite'],
                city=response[i]['address']['city'],
                zipcode=response[i]['address']['zipcode'],
                geo=Geo(
                    lat=response[i]['address']['geo']['lat'],
                    lng=response[i]['address']['geo']['lng']
                )
            ),
            phone=response[i]['phone'],
            website=response[i]['website'],
            company=Company(
                name=response[i]['company']['name'],
                catch_phrase=response[i]['company']['catchPhrase'],
                bs=response[i]['company']['bs']
            )
        ))

    return result


def get_posts(url: str, count: int):
    result: list[Post] = []
    request_url = build_request_url(url, ApiCategories.posts)

    response = get(request_url).json()

    for i in range(0, count):
        result.append(Post(
            user_id=response[i]['userId'],
            id=response[i]['id'],
            title=response[i]['title'],
            body=response[i]['body']
        ))

    return result


def get_comments(url: str, count: int):
    result: list[Comment] = []
    request_url = build_request_url(url, ApiCategories.comments)

    response = get(request_url).json()

    for i in range(0, count):
        result.append(Comment(
            post_id=response[i]['postId'],
            id=response[i]['id'],
            name=response[i]['name'],
            email=response[i]['email'],
            body=response[i]['body']
        ))

    return result


def get_albums(url: str, count: int):
    result: list[Album] = []
    request_url = build_request_url(url, ApiCategories.albums)

    response = get(request_url).json()

    for i in range(0, count):
        result.append(Album(
            user_id=response[i]['userId'],
            id=response[i]['id'],
            title=response[i]['title']
        ))

    return result


def get_photos(url: str, count: int):
    result: list[Photo] = []
    request_url = build_request_url(url, ApiCategories.photos)

    response = get(request_url).json()

    for i in range(0, count):
        result.append(Photo(
            album_id=response[i]['albumId'],
            id=response[i]['id'],
            title=response[i]['title'],
            url=response[i]['url'],
            thumbnail_url=response[i]['thumbnailUrl']
        ))

    return result



from dataclasses import dataclass


@dataclass
class Geo:
    lat: str
    lng: str


@dataclass
class Adress:
    street: str
    suite: str
    city: str
    zipcode: str
    geo: Geo


@dataclass
class Company:
    name: str
    catch_phrase: str
    bs: str



from setuptools import setup

setup(name='jsonplaceapi',
version='0.0.1',
description='Jsonplaceholder REST api',
url='https://github.com/defloppka/jsonplaceapi',
author='defloppka',
author_email='defloppka@gmail.com',
license='MIT',
packages=['jsonplaceapi'],
zip_safe=False)